package com.greatchn.authorization_server.config;

import com.greatchn.authorization_server.handler.AccDecisionHandler;
import com.greatchn.authorization_server.handler.AccDeniedHandler;
import com.greatchn.authorization_server.handler.AuthEntryPointHandler;
import com.greatchn.authorization_server.handler.MetadataSourceHandler;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.ObjectPostProcessor;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.NoOpPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.access.intercept.FilterSecurityInterceptor;

import javax.annotation.Resource;

/**
 * @AuthorizationServerConfig: 授权服务 config
 * @author: ZBoHang
 * @time: 2023/2/27 13:16
 */
@Configuration
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    @Resource
    private AuthEntryPointHandler authEntryPointHandler;
    @Resource
    private AccDeniedHandler accDeniedHandler;
    @Resource
    private UserDetailsService userDetailsHandler;
    @Resource
    private MetadataSourceHandler metadataSourceHandler;
    @Resource
    private AccDecisionHandler accDecisionHandler;
    @Resource
    private SecurityFilterConfig securityFilterConfig;

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
                // 允许postman等测试
                .csrf().disable()
                .sessionManagement()
                .sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and()
                .exceptionHandling()
                .accessDeniedHandler(this.accDeniedHandler)
                .authenticationEntryPoint(this.authEntryPointHandler)
                .and()
                .authorizeRequests()
                // .antMatchers("/login/**").permitAll()
                // .antMatchers("/admin/**").hasRole("ADMIN")
                // .antMatchers("/**")
                .anyRequest()
                .authenticated()
                // 自定义角色权限
                .withObjectPostProcessor(new ObjectPostProcessor<FilterSecurityInterceptor>() {
                    @Override
                    public <O extends FilterSecurityInterceptor> O postProcess(O object)  {
                        // 设置角色权限
                        object.setSecurityMetadataSource(metadataSourceHandler);
                        // 判定角色权限
                        object.setAccessDecisionManager(accDecisionHandler);
                        return object;
                    }
                })
                .and()
                // 跨域
                .cors().disable()
                .apply(this.securityFilterConfig)
        ;
    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        // 设置自定义的userDetailsService
        auth.userDetailsService(this.userDetailsHandler)
                .passwordEncoder(passwordEncoder());

    }

    @Bean
    public PasswordEncoder passwordEncoder() {
        // TODO: 2023/2/27 不加密
        return NoOpPasswordEncoder.getInstance();
    }
}
